function dots_y = CFU_model_sim(X,U,D) 
n_x = 10; % Number of vertically discretized control volumes, hence n_x + 2 nodes one each at the two boundaries x = 0 and x = L
% Control volumes start from the bottom. Hence x = 0 to x = L/n_x is the first control volume
scaling_factor = 1e10;
x = X;
u = U;

F_in = D(1);
ppm_in = D(2);
S = 0.01*F_in;
F_in = F_in/3600;
e_in = ppm_in*1e-6;

% Allocation of states
n_fb = x(1:n_x)*scaling_factor;
n_d = x(n_x+1:2*n_x)*scaling_factor;
n_lb = x(2*n_x+1:3*n_x)*scaling_factor;
alpha_l = x(3*n_x+1);
P = x(3*n_x+2)*1e5;
n_d_bottom = x(3*n_x+3)*scaling_factor;
alpha_gas_top = x(3*n_x+4);
alpha_oil_top = x(3*n_x+5);

% Dimensions
V = 3.2; % m^3 volume of the CFU main separation zone volume
R = 1.26/2; % Radius of CFU vessel in meter
L = V/pi/R^2;
dL = L/(n_x+2);

% Operating data
db = 200e-6; % Gas bubble size in meter
dd = 25e-6; % Oil droplet size in meter
V_fb = pi*db^3/6; % volume of the bubble at P_bottom
V_d = pi*dd^3/6;
Ec = (dd/db)^2; % Collision efficiency due to Flint and Howarth
k =  4e-11; % Loading rate constant [m^-3s^-1] Tuning parameter

% Fluid data and constants
rho_w = 1000;
rho_o = 900;
P_supply = 2e5;
%P_atm = 1e5;
M_g = 16e-3; % Molecular mass of methane in kg/mol
R_g = 8.314; % Universal gas cosntant
T = 300; % Operating Temperature in Kelvin
rho_g = P * M_g / R_g/T;

mu_w = 8.9e-4;
g = 9.81;
F_float = u(1)/3600;
F_out = u(2)/3600;

% Velocities
v_bulk = F_out/pi/R^2/alpha_l;
v_fb = (rho_w - rho_g)*g*db^2/18/mu_w -v_bulk;  % absolute up velocity
v_d  = -(rho_w - rho_o)*g*dd^2/18/mu_w + v_bulk; % absolute down velocity
v_lb = ((rho_w-rho_g)*V_fb+(rho_w-rho_o)*V_d)*g/3/pi/mu_w/((db^3+dd^3)^(1/3)) - v_bulk; % absolute up velocity

Split = 0.4082*(S-0.7)^2+0.8;
e_in_as = Split*e_in;
F_split_down = F_in*(1-e_in)/(1-e_in_as);
F_split_oil = F_in*e_in - F_split_down*e_in_as;


% droplets at bottom control volume below everything else. in this control
% volume we dont have differential equations for any thing else, no loading
% happening here

n_d_dot_bottom =   (v_d*n_d(1) - (F_out/pi/R^2)*n_d_bottom)/dL;

% Bottom boundary conditions
i = 1;
load_rate = (k*Ec*n_fb(i)*n_d(i));
n_fb_dot(i) =  (F_float*(P_supply/P)/V_fb/(pi*R^2)-n_fb(i)*v_fb)/dL  - load_rate;
n_d_dot(i) =   v_d*(n_d(i+1)-n_d(i))/dL - load_rate;
n_lb_dot(i) =  (0-n_lb(i)*v_lb)/dL   + load_rate;

% Differential equations for internal control volumes
for i = 2:n_x-1
load_rate = k*Ec*n_fb(i)*n_d(i);
n_fb_dot(i) =  v_fb*(n_fb(i-1)-n_fb(i))/dL    -load_rate;
n_d_dot(i) =   v_d*((n_d(i+1))-(n_d(i)))/dL   -load_rate;
n_lb_dot(i) =  v_lb*(n_lb(i-1)-n_lb(i))/dL    +load_rate;

end

% Top boundary conditions
i = n_x;
load_rate = k*Ec*n_fb(i)*n_d(i);
n_fb_dot(i) = v_fb*(n_fb(i-1)-n_fb(i))/dL  -load_rate;
n_d_dot(i) = (F_split_down*e_in_as/V_d/(pi*R^2)-n_d(i)*v_d)/dL - load_rate;
n_lb_dot(i) = v_lb*(n_lb(i-1)-n_lb(i))/dL  + load_rate;


F_reject = u(3)/3600;
F_reject_liq = F_reject*(1-alpha_gas_top);
alpha_l_dot = (F_in - F_out - F_reject_liq)/V;
V_l_dot = alpha_l_dot*V;
F_gas_out = alpha_gas_top*F_reject;
P_dot = (P*V_l_dot + (P_supply*F_float - P*F_gas_out))/V/(1-alpha_l);


alpha_gt_dot =    (V_fb*(v_fb*n_fb(n_x)+v_lb*n_lb(n_x))     - F_reject/(pi*R^2)*alpha_gas_top)/dL;
alpha_ot_dot =   (V_d*v_lb*n_lb(n_x) + (F_split_oil - F_reject*alpha_oil_top)/(pi*R^2))/dL;

ppm_out = n_d_bottom*V_d*1e6;
F_reject_water = F_reject*(1-alpha_gas_top - alpha_oil_top)*3600;
n_dot_flotation = P_supply*F_float*3600*M_g/R_g/T;

% Purging the differential equations
dots = [n_fb_dot'/scaling_factor;n_d_dot'/scaling_factor;n_lb_dot'/scaling_factor; alpha_l_dot; P_dot/1e5; n_d_dot_bottom/scaling_factor; alpha_gt_dot; alpha_ot_dot]; 
outputs = [e_in_as*1e6; ppm_out; P/1e5; F_reject_water; F_out*3600; n_dot_flotation; alpha_gas_top; alpha_oil_top; F_reject*3600];
dots_y = [dots;outputs];